#ifndef __ENCODE_H
#define __ENCODE_H

#include <stdio.h>
#include <vorbis/codec.h>

typedef void TIMER;
typedef long (*audio_read_func)(void *src, float **buffer, int samples);
typedef void (*progress_func)(char *fn, ogg_int64_t totalsamples, 
	ogg_int64_t samples, double time);
typedef void (*enc_end_func)(char *fn, double time, int rate, 
	ogg_int64_t samples, ogg_int64_t bytes);
typedef void (*enc_start_func)(char *fn, char *outfn, int bitrate, 
        float quality, int qset, int managed, int min_br, int max_br);
typedef void (*error_func)(char *errormessage);


void *timer_start(void);
double timer_time(void *);
void timer_clear(void *);
int create_directories(char *, int);

void add_TAG(char *flac_tag);
int tag_compare(const char *s1, const char *s2, int n);

void update_statistics_full(char *fn, ogg_int64_t total, ogg_int64_t done, double time);
void update_statistics_notime(char *fn, ogg_int64_t total, ogg_int64_t done, double time);
void update_statistics_null(char *fn, ogg_int64_t total, ogg_int64_t done, double time);
void start_encode_full(char *fn, char *outfn, int bitrate, float quality, int qset,
        int managed, int min, int max);
void start_encode_null(char *fn, char *outfn, int bitrate, float quality, int qset,
        int managed, int min, int max);
void final_statistics(char *fn, double time, int rate, ogg_int64_t total_samples,
		ogg_int64_t bytes);
void final_statistics_null(char *fn, double time, int rate, ogg_int64_t total_samples,
		ogg_int64_t bytes);
void encode_error(char *errmsg);

FILE *pipeopen ( const char* command, const char* filename );

typedef struct {
	char *arg;
	char *val;
} adv_opt;

typedef struct
{
	char **title;
	int title_count;
	char **artist;
	int artist_count;
	char **album;
	int album_count;
	char **comments;
	int comment_count;
	char **tracknum;
	int track_count;
	char **dates;
	int date_count;
	char **genre;
	int genre_count;
	char **lyrics;
	int lyrics_count;
	char **lyrics_language;
	int lyrics_language_count;
	adv_opt *advopt;
	int advopt_count;

	int copy_comments;
	int with_skeleton;
	int quiet;
	int rawmode;

	int raw_format;
	int raw_samplesize;
	int raw_samplerate;
	int raw_channels;
	int raw_endianness;

	char *namefmt;
	char *namefmt_remove;
	char *namefmt_replace;

	char *outfile;

	/* All 3 in kbps */
	int managed;
	int min_bitrate;
	int nominal_bitrate;
	int max_bitrate;

	int vbr_bitrate;
	/* Float from -1 to 1 (low->high) */
	float quality;
        int   quality_set;

	int resamplefreq;
	unsigned int converter;
	int downmix;
	float scale;

	int isutf8;
	unsigned int serial;
	unsigned int skeleton_serial;
	unsigned int kate_serial;
	int fixedserial;
	int padding;
/*	int padding; ** Removed - Win 7 issues */
	int rehuff;
	int FLAC_scale;
	int ignorelength;
} oe_options;

typedef struct
{
	vorbis_comment *comments;
	unsigned int serialno;
	unsigned int skeleton_serialno;
	unsigned int kate_serialno;

	audio_read_func read_samples;
	progress_func progress_update;
	enc_end_func end_encode;
	enc_start_func start_encode;
	error_func error;
	
	void *readdata;

	unsigned int total_samples_per_channel;
	int format;
	int channels;
	long rate;
	int samplesize;
	int endianness;
	unsigned int converter;
	int resamplefreq;
	double ratio;
	int copy_comments;
	int with_skeleton;
	int FLAC_scale;

	/* Various bitrate/quality options */
	int managed;
	int bitrate;
	int min_bitrate;
	int max_bitrate;
	float quality;
        int quality_set;
	adv_opt *advopt;
	int advopt_count;

	FILE *out;
	char *filename;
	char *infilename;
	int padding;
/*	int padding; ** Removed - Win 7 issues */
	int ignorelength;

	char *lyrics;
	char *lyrics_language;
} oe_enc_opt;


int oe_encode(oe_enc_opt *opt);

#endif /* __ENCODE_H */
